unit PlotPanel;
{
 *
 *  File:        	PlotPanel.pas (version 0.5)
 *  Purpose:    	Add a Plot component to Kylix, Delphi 5 or Delphi 6. 
 *			To replace more or less the Chart-component known from Delphi
 *  Author:      	Marien van Westen <m.c.van.westen@pl.hanze.nl> 
 *  Created:     	25 nov 2001
 *  Description:        Playing with the KOE and trying to port my Delphi
 *                      programs to Kylix I discovered that a lot of programs
 *                      used the Chart-component wich isn't included in Kylix.
 *                      Inspired by the MathPanel component by Mirko Patrizi I
 *                      started writing this component.
 *                      It allows up to 8 layers of graphs, and by using the
 *                      Freeze-method it is possible to produce flickerfree
 *                      animated plots.
 *
 *               
 *  Copyright (C) 2001  Marien van Westen
 *
 *  You are free to use and modify this code under the terms of
 *  the GNU General Public Licence version 2 or later.
 *  
 * 
 }


{$IFNDEF MSWINDOWS} 
  {$IFDEF WIN32} 
    {$DEFINE MSWINDOWS} 
  {$ENDIF} 
{$ENDIF} 


interface
{$IFDEF LINUX}
uses SysUtils, Types, Classes, QGraphics, QControls, QForms, QDialogs,
  QStdCtrls, QExtCtrls, QMenus, QTypes, Math;
{$ENDIF}
{$IFDEF MSWINDOWS}
  uses SysUtils, Types, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, Menus, Math;
{$ENDIF}

type
  TPlotMode = ( pmDot, pmLine, pmBar );

  Txyc = record
    x : Extended;
    y : Extended;
    c : TColor;
  end;

  TLayerOpt = record
     PlotMode :TPlotMode;
     PenWidth :Integer;
  end;

  TPlotPanel = class(TPanel)
  private
    // ***************************************************************
    //          PRIVATE VARIABLES
    // ***************************************************************
    LastXpixDrawn : Array [0..7] of Extended;
    LastYpixDrawn : Array [0..7] of Extended;
    //
    Xqnt : Extended;
    Yqnt : Extended;
    R2Width : integer;
    R2Height : integer;
    //
    fBackColor : Tcolor;
    fGridColor : Tcolor;
    //
    fMargin : integer;
    fMarginTop : integer;
    fMarginRight:Integer;
    fMarginBottom: Integer;
    fXMin : Extended;
    fXMax : Extended;
    fYMin : Extended;
    fYMax : Extended;
    //
    fXMarks : boolean;
    fXMarksFont : Tfont;
    fXMarksInterval : Extended;
    fYMarks : boolean;
    fYMarksFont : Tfont;
    fYMarksInterval : Extended;
    //
    fPlotMode : TPlotMode;
    fPlotPen : TPen;
    fRPlot, fRBorder : Trect;
    fClr : Boolean;
    fDataLength : Array [0..7] of Integer ;
    Datarray : Array[0..7] of Array of Txyc;
    fPlot : TBitmap;
    fBorder:TBitmap;
    fHeight : Integer;
    fFreeze : Boolean;
    FTitle : String;
    FLayerOpt : Array [0..7] of TLayerOpt;
    FLayerOptions : Boolean;
     // ***************************************************************
    //          ACCESS METHODS
    // ***************************************************************
    Procedure SetBackColor( value : Tcolor );
    Procedure SetGridColor( value : Tcolor );
    //
    Procedure SetMargin( value : integer );
    //
    Procedure SetXMin( value : Extended );
    Procedure SetXMax( value : Extended );
    Procedure SetYMin( value : Extended );
    Procedure SetYMax( value : Extended );
    //
    Procedure SetXMarks( value : boolean );
    Procedure SetXMarksFont( value : Tfont );
    Procedure SetXMarksInterval( value : Extended );
    Procedure SetYMarks( value : boolean );
    Procedure SetYMarksFont( value : Tfont );
    Procedure SetYMarksInterval( value : Extended );
    Procedure SetPlotMode( value : TPlotMode );
    Procedure SetPlotPen(value : TPen);
    Procedure SetTitle(value : String);
    Procedure PlotData( x, y : Extended; color : Tcolor;Layer:Integer );
    //
    Procedure AdjustR2Dimension;
    Procedure Teken;
    procedure DoTeken(Sender:TObject);
    Procedure SetLayerOptions(value:Boolean);
  protected
    { Protected declarations }
  public
    { Public declarations }
    Constructor Create( aOwner : Tcomponent ); override;
    Destructor Destroy; override;

    procedure Paint; override;
    //
    Procedure Clear;
    Procedure AddXY( x, y : Extended; color : Tcolor; Layer:Integer );overload;
    Procedure AddXY( x, y : Extended); overload;
    Procedure AutoScale(Layer:Integer);
    Procedure Freeze(_Boolean:Boolean);
    Procedure HideLayer(Layer:Integer);
    Procedure UnHideLayer(Layer:Integer);
    Function ConvertS2W(X,Y : Integer; var WX,WY : Extended): Boolean;
    Procedure LayerOptions(Layer:Integer;PlotMode:TPlotMode;PenWidth:Integer);
  published
    { Published declarations }
    Property GridColor : Tcolor read fGridColor write SetGridColor;
    Property BackColor : Tcolor read fBackColor write SetBackColor;
    Property Margin : integer read fMargin write SetMargin;
    //
    Property XMin : Extended read fXmin write SetXMin;
    Property XMax : Extended read fXmax write SetXMax;
    Property YMin : Extended read fYmin write SetYMin;
    Property YMax : Extended read fYmax write SetYMax;
    //
    Property XMarks : boolean read fXMarks write SetXMarks;
    Property XMarksFont : Tfont read fXMarksFont write SetXMarksFont;
    Property XMarksInterval : Extended read fXMarksInterval write SetXMarksInterval;
    Property YMarks : boolean read fYMarks write SetYMarks;
    Property YMarksFont : Tfont read fYMarksFont write SetYMarksFont;
    Property YMarksInterval : Extended read fYMarksInterval write SetYMarksInterval;
    //
    Property PlotMode : TPlotMode read fPlotMode write SetPlotMode;
    Property PlotPen : TPen read fPlotPen write SetPlotPen;
    Property Title : String read FTitle write SetTitle;
    Property LayerOption : Boolean read FLayerOptions write SetLayerOptions;
    //
  end;

procedure Register;

implementation

//
Procedure TPlotPanel.SetBackColor( value : Tcolor );
begin
     fBackColor := value;
     Teken;
end;

//
Procedure TPlotPanel.SetGridColor( value : Tcolor );
begin
     fGridColor := value;
     Teken;
end;

//
Procedure TPlotPanel.SetMargin( value : integer );
begin
     fMargin := value;
     Teken;
end;

//
Procedure TPlotPanel.SetXMin( value : Extended );
begin
     fXMin := value;
     if fXMin>=fXMax then SetXMax(fXMin+1);
     Teken;
end;

//
Procedure TPlotPanel.SetXMax( value : Extended );
begin
     fXMax := value;
     if fXMin>=fXMax then SetXMin(fXMax-1);
     Teken;
end;

//
Procedure TPlotPanel.SetYMin( value : Extended );
begin
     fYMin := value;
     if fYMin>=fYMax then SetYMax(fYMin+1);
     Teken;
end;

//
Procedure TPlotPanel.SetYMax( value : Extended );
begin
     fYMax := value;
     if fYMin>=fYMax then SetYMin(fYMax-1);
     Teken;
end;

//
Procedure TPlotPanel.SetXMarks( value : boolean );
begin
     fXMarks := value;
     Teken;
end;

//
Procedure TPlotPanel.SetXMarksFont( value : Tfont );
begin
     fXMarksFont.Assign( value );
     Teken;
end;

Procedure TPlotPanel.SetPlotPen( value : TPen );
begin
     fPlotPen.Assign( value );
     Teken;
end;

//
Procedure TPlotPanel.SetXMarksInterval( value : Extended );
begin
  if value=0 then value:=1;
  fXMarksInterval:=value;
  Teken;
end;

//
Procedure TPlotPanel.SetYMarks( value : boolean );
begin
     fYMarks := value;
     Teken;
end;

//
Procedure TPlotPanel.SetYMarksFont( value : Tfont );
begin
     fYMarksFont.Assign( value );
     Teken;
end;

//
Procedure TPlotPanel.SetYMarksInterval( value : Extended );
begin
 if value=0 then value:=1;
 fYMarksInterval:=value;
 Teken;
end;

//
Procedure TPlotPanel.SetTitle( value : String );
begin
     FTitle := value ;
     Teken;
end;

//
Procedure TPlotPanel.SetPlotMode( value : TPlotMode );
begin
     fPlotMode := value;
     Teken;
end;

//
Procedure TPLotPanel.SetLayerOptions(value:Boolean);
begin
 if value=false then fLayerOptions:=False;
end;

//
Procedure TPlotPanel.AdjustR2Dimension;
begin
     // Width and Height (pixels) of the cartesian area
     R2Width  := width - ( fMargin+fMarginRight );
     R2Height := height - ( fMarginTop+fMarginBottom );
     // Pixels per numeric unit
     Xqnt := R2Width / (fXMax - fXMin);
     Yqnt := R2Height / (fYMax - fYMin);
end;


// 
Constructor TPlotPanel.Create;
var
  i : Integer;
begin
     inherited Create( aOwner );
     {$IFDEF MSWINDOWS}
     Parent:=TWinControl(Aowner);
     {$ENDIF}
     // Global panel default settings
     Height :=200;
     Width  :=300;
     Caption :=' ';
     // Graph-backgound and axis colors
     fBackColor := clWhite;
     fGridColor := clBlack;
     fPlotMode:=pmLine;
     //fMarginColor:=clButton;
     // Default distances between graph-edges and panel borders  
     fMargin := 15;
     fMarginTop := fMargin;
     fMarginBottom:=fMargin;
     fMarginRight:=fMargin;
     // Numerical starting ranges   
     fXMin := 0;
     fXMax := 10;
     fYMin := 0;
     fYMax := 10;
     // Values referred to the axis aspect   
     fXMarks := false;
     fXMarksFont := Tfont.Create;
     fXMarksInterval := 1;
     fYMarks := false;
     fYMarksFont := Tfont.Create;
     fYMarksInterval := 1;
     //
     fPlotPen:=TPen.Create;
     fPlotPen.OnChange:=DoTeken;
     fLayerOptions:=False;
     fXMarksFont.OnChange:=DoTeken;
     fYMarksFont.OnChange:=DoTeken;
     //
     for i:=0 to 7 do
       begin
       fLayerOpt[i].PlotMode:=fPlotMode;
       fLayerOpt[i].PenWidth:=1;
       fDataLength[i]:=1;
       SetLength(Datarray[i],1);
       end;
     fPlot:=TBitmap.Create;
     fBorder:=TBitmap.Create;
     Teken;
end;


// 
Destructor TPlotPanel.Destroy;
begin
     If assigned( fXMarksFont ) then
     begin
          fXMarksFont.Free;
          fXMarksFont := nil;
     end;
     If assigned( fYMarksFont ) then
     begin
          fYMarksFont.Free;
          fYMarksFont := nil;
     end;
     if assigned (fPlotPen) then
     begin
        fPlotPen.Free;
        fPlotPen:=nil;
     end;
     //
     if assigned (fPlot) then
     begin
        fPlot.Free;
        fPlot:=nil;
     end;
     //
     if assigned (fBorder) then
     begin
        fBorder.Free;
        fBorder:=nil;
     end;
     //
     inherited Destroy;
end;

//
Procedure TPlotPanel.Paint;
begin
  inherited Paint;
  Teken;
end; 

//

Procedure TPlotPanel.DoTeken(Sender:TObject);
begin
  //Teken is without parameter so we do it this way
  Teken;
end; 

Procedure TPlotPanel.Teken;
Var
    XStr, YStr : string;
    i,j : Integer;
    tmp : Boolean;
    XStart, X : Extended;
    YStart, Y : Extended;
    ACanvas : TCanvas;
begin
     //
     if (csDesigning in ComponentState) then ACanvas:=Canvas
                                         else ACanvas:=fBorder.Canvas;
     // Adjust Margins
     if Length(fTitle)<>0 then fMarginTop:=fMargin+abs(Font.Height)+2
                   else fMarginTop:=fMargin;
     if fXMarks    then fMarginBottom:=fMargin+abs(fXMarksFont.Height)+2
                   else fMarginBottom:=fMargin;
     if fyMarks    then fMarginRight:=fMargin+20+2
                   else fMarginRight:=fMargin;
     // Refresh proportional value
     AdjustR2Dimension;
     // Draws the window for the graph
     if ACanvas=fBorder.Canvas then
        begin
        fBorder.Width:=Width-2*BevelWidth;
        fBorder.Height:=Height-2*BevelWidth;
        end;

     fPlot.Width:=Width;
     fPlot.Height:=Height;
     // fRBorder is the whole visible area of the Panel (Border)
     fRBorder.Left:=2*BevelWidth;
     fRBorder.Top:=2*BevelWidth;
     fRBorder.Right:=Width-2*BevelWidth;
     fRBorder.Bottom:=Height-2*BevelWidth;
     ACanvas.Brush.Color:=Color;
     ACanvas.FillRect(fRBorder);
     // fRPlot is the part where the plotting is done
     fRPlot.Left := fMargin;
     fRPlot.Top := fMarginTop;
     fRPlot.Right := Width-fMarginRight;
     fRPlot.Bottom :=Height-fMarginBottom;
     fHeight:=fRPlot.Bottom;

     ACanvas.Brush.Color := fBackColor;
     ACanvas.FillRect( fRPlot );
     // Draws cartesian axis
     ACanvas.Pen.Color := fGridColor;
     ACanvas.Pen.Width:=1;

     // Y-axis (vertical) lines
     Xstart:=fXmarksInterval*Round(0.4999+(fXmin/fXMarksInterval));
     X:=Xstart;
     repeat

       ACanvas.moveTo( Round((X-FXMin)*Xqnt)+FMargin,
                            fMarginTop);
       ACanvas.lineto( Round((X-FXMin)*Xqnt)+FMargin,
                            height-fMarginBottom);
       X:=X +FXMarksInterval;
     until X>FXMax*1.000001;  // something goes wrong with rounding ???

     // X-axis (horizontal) lines
     Ystart:=fYmarksInterval*Round(0.4999+(fYmin/fYMarksInterval));
     Y:=Ystart;
     repeat
       ACanvas.moveTo( fMargin ,
                       Height-FMarginBottom-Round((Y-FYMin)*Yqnt));
       ACanvas.lineto( width -fMarginRight ,
                       Height-FMarginBottom-Round((Y-FYMin)*Yqnt));
       Y:=Y +FYMarksInterval;
     until Y>FYMax*1.000001;

    //
    if FTitle<>'' Then
       begin
       ACanvas.Brush.Color := Color;
       ACanvas.Font.Assign( Font); //fTitleFont );
       ACanvas.Textout((width-Canvas.TextWidth(FTitle)) div 2,
                        BevelWidth*2+fMargin div 2,
                        FTitle);
       end;
    //
    If fXMarks then
     begin
          // ***********************
          // WRITES X AXIS VALUES

          // Sets X-axis color
          ACanvas.Brush.Color := Color;
          ACanvas.Font.Assign( fXMarksFont );
          // Write X values
          X:=Xstart;
          repeat
            if X<1000
               then XStr := FormatFloat('0.##', X)
               else XStr := FormatFloat('0.##E-00', X);
               ACanvas.TextOut((-ACanvas.TextWidth(Xstr) div 2)+ Round((X-FXMin)*Xqnt)+FMargin ,
                               height - fMarginBottom +3,
                               XStr );
            X:=X +FXMarksInterval;
          until X>FXMax*1.000001;
     end;

   //
   If fYMarks then
     begin
          // ***********************
          // WRITES Y AXIS VALUES

          // Sets Y-axis color
          ACanvas.Brush.Color := Color;
          ACanvas.Font.Assign( fYMarksFont );
          // Writes Y values
          Y:=YStart;
          repeat
            if Y< 1000
               then YStr := FormatFloat('0.##',Y)
               else YStr := FormatFloat('0.##E-00',Y);
               ACanvas.TextOut( width + 5-fMarginRight,
                                Height-FMarginBottom-ACanvas.TextHeight(Ystr) div 2-Round((Y-FYMin)*Yqnt),
                                YStr );
            Y:=Y +FYMarksInterval;
          until Y>FYMax*1.000001;
     end;

   if (csDesigning in ComponentState) then Exit;
      
   fPlot.Canvas.CopyRect(fRPlot,ACanvas,fRPlot);
      
   // Plot eventuele data
      tmp:=fFreeze;
      fFreeze:=true;
        for j:=0 to 7 do
          begin
          if fDataLength[j]>1 then
             begin
             fClr:=True;
             for i:=1 to fDataLength[j]-1 do
               PlotData(Datarray[j,i].X, Datarray[j,i].Y, Datarray[j,i].c,j);
             end;
           end;
     fFreeze:=tmp;
     fBorder.Canvas.CopyRect(fRPlot,fPlot.Canvas,fRPlot);
    if not fFreeze then Canvas.CopyRect(fRBorder,fBorder.Canvas,fRBorder);
  //
end;

//
Procedure TPlotPanel.Freeze(_Boolean:Boolean);
begin
if _Boolean then fFreeze:=true
         else begin
              fFreeze:=False;
              fBorder.Canvas.CopyRect(fRPlot,fPlot.Canvas,fRPlot);
              Canvas.CopyRect(fRBorder,fBorder.Canvas,fRBorder);
              end;
end;

//
procedure TPlotPanel.Clear;
var
  i : Integer;
begin
     fClr:=True;
     for i:=0 to 7 do
       begin
       fDataLength[i]:= 1;
       Datarray[i]:=Copy(Datarray[i],0,1);
       SetLength(Datarray[i],1);
       end;
     Teken;
end;

//
Procedure TPlotPanel.AutoScale(Layer:Integer);
var
  i : Integer;
  x, y, Range,f : Extended;
begin

  if fDataLength[Layer]>2 then
         begin
           fXMin:=Datarray[Layer,1].X;
           fXMax:=fXMin*1.001;
           fYMin:=Datarray[Layer,1].Y;
           fYMax:=fXMin*1.001;

           if Datarray[Layer,2].X > fXMin then fXMax:=Datarray[Layer,2].x;
           if Datarray[Layer,2].x < fXMin then begin
                                           fxMax:=fXMin;
                                           fXMin:=Datarray[Layer,2].x;
                                         end;
           if Datarray[Layer,2].y > fyMin then fyMax:=Datarray[Layer,2].y;
           if Datarray[Layer,2].y < fyMin then begin
                                           fyMax:=fyMin;
                                           fyMin:=Datarray[Layer,2].y;
                                         end;
           for i:=3 to fDataLength[Layer]-1 do
             begin
             x:=Datarray[Layer,i].x;
             y:=Datarray[Layer,i].y;
             if x>fXMax then fXMax:=x;
             if x<fXMin then fXMin:=x;
             if y>fYMax then fYMax:=y;
             if y<fYMin then fYMin:=y;
             end;

         Range:=fXMax-fXMin;
         f:=1;
         While Range>100 do
            begin
            Range:=Range/10;
            f:=f/10;
            end;
         While Range<10 do
            begin
            Range:=Range*10;
            f:=f*10;
            end;
         fXMarksInterval:=Round(0.4999+Range/10)/f;
        // fXMin:=fXMarksInterval*Round(-0.5+fXMin/fXMarksInterval);
        // fXMax:=fXMarksInterval*Round(0.4999+fXMax/fXMarksInterval);

         Range:=fYMax-fYMin;
         f:=1;
         While Range>100 do
            begin
            Range:=Range/10;
            f:=f/10;
            end;
         While Range<10 do
            begin
            Range:=Range*10;
            f:=f*10;
            end;
         fYMarksInterval:=Round(0.4999+Range/10)/f;
        // fYMin:=fYMarksInterval*Round(-0.5+fYMin/fYMarksInterval);
        // fYMax:=fYMarksInterval*Round(0.4999+fYMax/fYMarksInterval);

         Teken;
     end; //if
end ;

//
Procedure TPlotPanel.AddXY( x, y : Extended; color : Tcolor; Layer:Integer );
begin
      Layer:=Layer and 7;
      if fDatalength[Layer]=1 then fClr:=True;
      Inc(fDataLength[Layer]);
      SetLength(Datarray[Layer],fDataLength[Layer]);
      Datarray[Layer,fDataLength[Layer]-1].X:=x;
      Datarray[Layer,fDataLength[Layer]-1].y:=y;
      Datarray[Layer,fDataLength[Layer]-1].c:=color;
      Plotdata(x,y,color,Layer);
      if not fFreeze then Canvas.CopyRect(fRBorder,fBorder.Canvas,fRBorder);
end;

//
Procedure TPlotPanel.AddXY( x, y : Extended);
var
  Color: TColor;
  Layer: Integer;
begin
    Layer:=0;
    Color:=fPlotPen.Color;
    FLayerOptions:=False;
    if fDatalength[Layer]=1 then fClr:=True;
      Inc(fDataLength[Layer]);
      SetLength(Datarray[Layer],fDataLength[Layer]);
      Datarray[Layer,fDataLength[Layer]-1].X:=x;
      Datarray[Layer,fDataLength[Layer]-1].y:=y;
      Datarray[Layer,fDataLength[Layer]-1].c:=color;
      Plotdata(x,y,Color,Layer);
      if not fFreeze then Canvas.CopyRect(fRBorder,fBorder.Canvas,fRBorder);
end;

//
Procedure TPlotPanel.PlotData( x, y : Extended; color : Tcolor;Layer:Integer );
var
  Punt : TRect;
  l,b : Integer;
  s, s2 : Integer;
begin
      l:=fMargin;
      b:=Height-fMarginBottom;
      s:=fPlotPen.Width div 2;
      s2:=fPlotPen.Width;   
      fPlot.Canvas.Pen:=fPlotPen;
      fPlot.Canvas.Pen.Color:=color;
      if FLayerOptions=True then
         begin
         fPlotMode:=fLayerOpt[Layer].PlotMode;
         if fPlotMode=pmDot then
             begin
             s := (fLayerOpt[Layer].PenWidth) div 2 ;
             s2:=  fLayerOpt[Layer].PenWidth;
             end
         else fPlot.Canvas.Pen.Width:=fLayerOpt[Layer].PenWidth;
         end;

      Case fPlotMode of
      pmDot:
           begin

              fPlot.Canvas.Brush.Color := color;
              Punt.Left:=Round(-s+l+(x - fXMin)* Xqnt);
              Punt.Bottom:=+s+b - Round((y - fYMin)* Yqnt);
              Punt.Right:=-s+s2+l+ Round((x - fXMin)* Xqnt);
              Punt.Top:=b +s-s2 - Round((y - fYMin)* Yqnt);
              fPlot.Canvas.FillRect(Punt);
              fPlot.Canvas.Brush.Color := fBackColor;
           end;
      pmLine:
           begin
             if fClr then
                begin
                  fPlot.Canvas.MoveTo(l+Round((x - fXMin)* Xqnt),
                                      b - Round((y - fYMin)* Yqnt));
                  fClr:=False;
                end
                else
                begin
                fPlot.Canvas.MoveTo( Round(LastXpixDrawn[Layer]),
                                     Round(LastYpixDrawn[Layer]));
                fplot.Canvas.LineTo(l+Round((x - fXMin)* Xqnt) ,
                                    Round(b-(y-fYMin)* Yqnt));
                end;
             LastXpixDrawn[Layer] :=l+(x - fXMin)* Xqnt;
             LastYpixDrawn[Layer] :=b -(y - fYMin)* Yqnt;
           end;
      pmBar:
           begin
              fPlot.Canvas.MoveTo(l+Round((x - fXMin)* Xqnt) ,
                                  b - Round(-fYMin* Yqnt));
              fPlot.Canvas.LineTo(l+Round((x - fXMin)* Xqnt),
                                  b - Round((y-fYmin)*Yqnt));
           end;
       end; // case
     if not fFreeze then fBorder.Canvas.CopyRect(fRPlot,fPlot.Canvas,fRPlot);
end;

//
Procedure TPlotPanel.HideLayer(Layer:Integer);
begin
  Layer:=Layer and 7;
  fDataLength[Layer]:= 1;
  Teken;
end;

//
Procedure TPlotPanel.UnHideLayer(Layer:Integer);
begin
  Layer:=Layer and 7;
  fDataLength[Layer]:= Length(Datarray[Layer]);
  Teken;
end;

//
Function TPlotPanel.ConvertS2W(X,Y : Integer; var WX,WY : Extended): Boolean;
begin
 Result:=True;
 X:=X-fMargin;
 Y:=Height-fMarginBottom-Y;
 WX:=fXMin + x/Xqnt;
 WY:=FYMin + y/Yqnt;
 if WX<fXMin then Result:=False;
 if WX>fXMax then Result:=False;
 if WY<fYMin then Result:=False;
 if WY>fYMax then Result:=False;
end;

Procedure TPlotPanel.LayerOptions(Layer:Integer;PlotMode:TPlotMode;PenWidth:Integer);
begin
 FLayerOptions:=True;
 Layer:=Layer and 7;
 fLayerOpt[Layer].PlotMode:=Plotmode;
 fLayerOpt[Layer].PenWidth:=PenWidth;
end;

//
procedure Register;
begin
  RegisterComponents('Samples', [TPlotPanel]);
end;


end.





